#!/bin/sh

# This fix is used for plesk 9.0-9.2.3 to resolve a security problem
# described in:
#    http://plesk-process.parallels.com/TargetProcess2/Project/QA/Bug/View.aspx?BugID=42479
#
# and KB article:
#    http://kb.parallels.com/en/113818
#
# and Here:
#    http://kb.parallels.com/en/113814
#
# It's changing phppath to segregate directory instead of /usr/bin
# thus resolve a security problem.
#

die()
{
	echo "ERROR: $*"
	exit 1
}

quit()
{
	echo "NOTE: $*"
	exit 0
}

parse_conf()
{
	conffile="/etc/psa/psa.conf"

	[ ! -f "$conffile" ] && die "File not found: '$conffile'"

	eval `cat $conffile | \
		grep -v '^ ' | grep -v '^$' | grep -v '^#' | \
		awk '{print $1"="$2}'`

	[ -z "$HTTPD_VHOSTS_D" ] && die "Unable to read the conf file: '$conffile'"
}

init_vars()
{
	parse_conf

	src_path="`dirname $0`"

	arch="`uname -m | awk '/64/ {print "x86_64"; exit;} {print "i386";}'`"

	chroot_d="`readlink -f $HTTPD_VHOSTS_D/..`"
	chroot_d="$chroot_d/cgi-bin/cgi_wrapper"

	wrapper_bin="$chroot_d/cgi_wrapper"
	wrapper_src="$src_path/$arch/cgi_wrapper"

	php_path="/usr/bin/php-cgi /usr/bin/php5-cgi /usr/bin/php4-cgi /srv/www/cgi-bin/php /usr/libexec/php5-cgi /usr/libexec/php4-cgi /usr/bin/php"
	php_cgi_bin=""
	for i in $php_path; do
		if [ -x "$i" ]; then
			php_cgi_bin="$i"
			break
		fi
	done
	[ -z "$php_cgi_bin" ] && die "Unable to find php binary."
}

install_wrapper()
{
	echo -n "Trying to install php cgi wrapper.. "

	if [ -x "$wrapper_bin" ]; then
		echo "already installed."
		echo "Please remove $wrapper_bin and try again if required."
		return
	fi

	mkdir -p $chroot_d
	chown root:0 $chroot_d
	chmod 0755   $chroot_d

	cp -f $wrapper_src $wrapper_bin
	chown root:0 $wrapper_src
	chmod 0755   $wrapper_src

	install_php_config

	echo "done"

	echo -n "Trying to reconfigure web server.. "
	$PRODUCT_ROOT_D/admin/sbin/websrvmng --reconfigure-all
	echo "done"

	echo -n "Trying to restart web server.. "
	$PRODUCT_ROOT_D/admin/sbin/websrvmng --stop
	sleep 2
	$PRODUCT_ROOT_D/admin/sbin/websrvmng --start
	echo "done"
}

install_php_config()
{
	local cgi_conffile="$HTTPD_INCLUDE_D/php_cgi.conf"

	for f in `find $HTTPD_INCLUDE_D -type f`; do
		grep -q '^scriptAlias' $f || continue

		echo -n "pathing the file '$f'.. "
		cat $f | awk '{ \
			if ((($1 ~ /^scriptAlias/) && ($2 ~ /phppath/)) \
				|| (($1 ~ /^Action/) && ($2 ~ /php-script/))) \
			{ \
				print "#"$0; \
				next; \
			} \
		}' > ${f}.new
		mv -f ${f}.new ${f}
	done

	echo "scriptAlias /phppath/ \"$chroot_d/\"" > $cgi_conffile
	echo "Action php-script /phppath/`basename $wrapper_bin`" >> $cgi_conffile

	chown root:root $cgi_conffile
	chmod 0644 $cgi_conffile

	sed -i -e "s|CGI_PHP_BIN\(.*\)|CGI_PHP_BIN $wrapper_bin|g" $conffile
}

init_vars
install_wrapper

exit 0
